@@ -1,5 +1,4 @@
 # -*- python-mode -*-
-# -*- coding: UTF-8 -*-
 
 ## Copyright (C) 2012-2013  Daniel Pavel
 ##
@@ -17,8 +16,6 @@
 ## with this program; if not, write to the Free Software Foundation, Inc.,
 ## 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 from collections import namedtuple
 
 from .common import NamedInts as _NamedInts
@@ -330,7 +327,7 @@ _D(
     settings=[_FS.new_fn_swap()],
 )
 _D('Craft Advanced Keyboard', codename='Craft', protocol=4.5, wpid='4066', btid=0xB350)
-_D('MX Keys Keyboard', codename='MK Keys', protocol=4.5, wpid='408A', btid=0xB35B)
+_D('MX Keys Keyboard', codename='MX Keys', protocol=4.5, wpid='408A', btid=0xB35B)
 _D(
     'Wireless Keyboard S510',
     codename='S510',
@@ -345,6 +342,13 @@ _D(
     wpid='0065',
     registers=(_R.battery_status, ),
 )
+_D(
+    'G915 TKL LIGHTSPEED Wireless RGB Mechanical Gaming Keyboard',
+    codename='G915 TKL',
+    protocol=4.2,
+    wpid='408E',
+    usbid=0xC343,
+)
 
 # Mice
 
@@ -458,7 +462,7 @@ _D(
     wpid='406D',
     settings=[
         _FS.hires_smooth_invert(),
-        _FS.hires_smooth_resolution(),
+        #        _FS.hires_smooth_resolution(),
         _FS.pointer_speed(),
     ]
 )
@@ -483,7 +487,7 @@ _D(
     wpid='404A',
     settings=[
         _FS.hires_smooth_invert(),
-        _FS.hires_smooth_resolution(),
+        #        _FS.hires_smooth_resolution(),
     ],
 )
 _D(
@@ -510,7 +514,7 @@ _D(
     btid=0xb012,
     settings=[
         _FS.hires_smooth_invert(),
-        _FS.hires_smooth_resolution(),
+        #        _FS.hires_smooth_resolution(),
     ],
 )
 
@@ -522,7 +526,7 @@ _D(
     btid=0xb019,
     settings=[
         _FS.hires_smooth_invert(),
-        _FS.hires_smooth_resolution(),
+        #        _FS.hires_smooth_resolution(),
         _FS.gesture2_gestures(),
     ],
 )
@@ -543,6 +547,8 @@ _D(
     codename='G700',
     protocol=1.0,
     wpid='1023',
+    usbid=0xc06b,
+    interface=1,
     registers=(
         _R.battery_status,
         _R.three_leds,
@@ -569,6 +575,7 @@ _D(
     ],
 )
 
+_D('G102 Lightsync Mouse', codename='G102', usbid=0xc092, interface=1)
 _D('G403 Gaming Mouse', codename='G403', usbid=0xc082)
 _D('G502 Hero Gaming Mouse', codename='G502 Hero', usbid=0xc08d)
 _D('G703 Lightspeed Gaming Mouse', codename='G703', usbid=0xc087)
@@ -577,6 +584,16 @@ _D('G900 Chaos Spectrum Gaming Mouse', codename='G900', usbid=0xc081)
 _D('G903 Lightspeed Gaming Mouse', codename='G903', usbid=0xc086)
 _D('G903 Hero Gaming Mouse', codename='G903 Hero', usbid=0xc091)
 _D('GPro Gaming Mouse', codename='GPro', usbid=0xc088)
+_D(
+    'PRO X Wireless',
+    kind='mouse',
+    codename='PRO X',
+    protocol=4.2,
+    wpid='4093',
+    usbid=0xc094,
+)
+
+_D('M500S Mouse', codename='M500S', usbid=0xc093, interface=1)
 
 _D(
     'LX5 Cordless Mouse',
@@ -722,3 +739,14 @@ _D(
 # Some exotics...
 
 _D('Fujitsu Sonic Mouse', codename='Sonic', protocol=1.0, wpid='1029')
+
+# Headset
+
+_D(
+    'Logitech PRO X Wireless Gaming Headset',
+    codename='PRO Wireless Headset',
+    protocol=2.0,
+    interface=3,
+    kind=_DK.headset,
+    usbid=0x0aba
+)
