@@ -1,9 +1,6 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 import errno as _errno
 
 from logging import INFO as _INFO
-from logging import WARNING as _WARNING
 from logging import getLogger
 from typing import Optional
 
@@ -15,20 +12,22 @@ from . import descriptors as _descriptors
 from . import hidpp10 as _hidpp10
 from . import hidpp20 as _hidpp20
 from .common import strhex as _strhex
-from .i18n import _
 from .settings_templates import check_feature_settings as _check_feature_settings
 
 _log = getLogger(__name__)
 del getLogger
 
 _R = _hidpp10.REGISTERS
+_IR = _hidpp10.INFO_SUBREGISTERS
+
+KIND_MAP = {kind: _hidpp10.DEVICE_KIND[str(kind)] for kind in _hidpp20.DEVICE_KIND}
 
 #
 #
 #
 
 
-class Device(object):
+class Device:
 
     read_register = _hidpp10.read_register
     write_register = _hidpp10.write_register
@@ -38,11 +37,14 @@ class Device(object):
         self.receiver = receiver
         self.may_unpair = False
         self.isDevice = True  # some devices act as receiver so we need a property to distinguish them
+        self.handle = None
+        self.path = None
+        self.product_id = None
 
         if receiver:
             assert number > 0 and number <= receiver.max_devices
         else:
-            assert number == 0
+            assert number == 0xFF
         # Device number, 1..6 for unifying devices, 1 otherwise.
         self.number = number
         # 'device active' flag; requires manual management.
@@ -87,10 +89,6 @@ class Device(object):
         # See `add_notification_handler`
         self._notification_handlers = {}
 
-        self.handle = None
-        self.path = None
-        self.product_id = None
-
         # if _log.isEnabledFor(_DEBUG):
         #     _log.debug("new Device(%s, %s, %s)", receiver, number, link_notification)
 
@@ -107,36 +105,15 @@ class Device(object):
                 self._kind = _hidpp10.DEVICE_KIND[kind]
             else:
                 # Not a notification, force a reading of the wpid
-                pair_info = self.receiver.read_register(_R.receiver_info, 0x20 + number - 1)
-                if pair_info:
-                    # may be either a Unifying receiver, or an Unifying-ready
-                    # receiver
-                    self.wpid = _strhex(pair_info[3:5])
-                    kind = ord(pair_info[7:8]) & 0x0F
-                    self._kind = _hidpp10.DEVICE_KIND[kind]
-                elif receiver.ex100_27mhz_wpid_fix:
-                    # 27Mhz receiver, fill extracting WPID from udev path
-                    self.wpid = _hid.find_paired_node_wpid(receiver.path, number)
-                    if not self.wpid:
-                        _log.error('Unable to get wpid from udev for device %d of %s', number, receiver)
-                        raise _base.NoSuchDevice(number=number, receiver=receiver, error='Not present 27Mhz device')
-                    kind = self.get_kind_from_index(number, receiver)
-                    self._kind = _hidpp10.DEVICE_KIND[kind]
-                else:
-                    # unifying protocol not supported, must be a Nano receiver
-                    device_info = self.receiver.read_register(_R.receiver_info, 0x04)
-                    if device_info is None:
-                        _log.error('failed to read Nano wpid for device %d of %s', number, receiver)
-                        raise _base.NoSuchDevice(number=number, receiver=receiver, error='read Nano wpid')
-                    self.wpid = _strhex(device_info[3:5])
-                    self._power_switch = '(' + _('unknown') + ')'
+                self.online = True
+                self.update_pairing_information()
 
             # the wpid is necessary to properly identify wireless link on/off
             # notifications also it gets set to None on this object when the
             # device is unpaired
             assert self.wpid is not None, 'failed to read wpid: device %d of %s' % (number, receiver)
 
-            self.path = _hid.find_paired_node(receiver.path, number, _base.DEFAULT_TIMEOUT)
+            self.path = _hid.find_paired_node(receiver.path, number, 1)
             try:
                 self.handle = _hid.open_path(self.path) if self.path else None
             except Exception:  # maybe the device wasn't set up
@@ -149,18 +126,15 @@ class Device(object):
 
             self.descriptor = _descriptors.get_wpid(self.wpid)
             if self.descriptor is None:
-                # Last chance to correctly identify the device; many Nano
-                # receivers do not support this call.
-                codename = self.receiver.read_register(_R.receiver_info, 0x40 + self.number - 1)
+                # Last chance to correctly identify the device; many Nano receivers do not support this call.
+                codename = self.receiver.device_codename(self.number)
                 if codename:
-                    codename_length = ord(codename[1:2])
-                    codename = codename[2:2 + codename_length]
-                    self._codename = codename.decode('ascii')
+                    self._codename = codename
                     self.descriptor = _descriptors.get_codename(self._codename)
         else:
             self.path = info.path
             self.handle = _hid.open_path(self.path)
-            self.online = True
+            self.online = None  # a direct connected device might not be online (as reported by user)
             self.product_id = info.product_id
             self.bluetooth = info.bus_id == 0x0005
             self.descriptor = _descriptors.get_btid(self.product_id
@@ -197,12 +171,12 @@ class Device(object):
         if not self._codename:
             if self.online and self.protocol >= 2.0:
                 self._codename = _hidpp20.get_friendly_name(self)
+                if not self._codename:
+                    self._codename = self.name.split(' ', 1)[0] if self.name else None
             elif self.receiver:
-                codename = self.receiver.read_register(_R.receiver_info, 0x40 + self.number - 1)
+                codename = self.receiver.device_codename(self.number)
                 if codename:
-                    codename_length = ord(codename[1:2])
-                    codename = codename[2:2 + codename_length]
-                    self._codename = codename.decode('utf-8')
+                    self._codename = codename
                 elif self.protocol < 2.0:
                     self._codename = '? (%s)' % (self.wpid or self.product_id)
         return self._codename if self._codename else '?? (%s)' % (self.wpid or self.product_id)
@@ -212,40 +186,66 @@ class Device(object):
         if not self._name:
             if self.online and self.protocol >= 2.0:
                 self._name = _hidpp20.get_name(self)
-        return self._name or self.codename or ('Unknown device %s' % (self.wpid or self.product_id))
+        return self._name or self._codename or ('Unknown device %s' % (self.wpid or self.product_id))
 
     @property
     def unitId(self):
         if not self._unitId:
             if self.online and self.protocol >= 2.0:
-                self._unitId, self._modelId, self._tid_map = _hidpp20.get_ids(self)
-                if _log.isEnabledFor(_INFO) and self._serial and self._serial != self._unitId:
-                    _log.info('%s: unitId %s does not match serial %s', self, self._unitId, self._serial)
+                ids = _hidpp20.get_ids(self)
+                if ids:
+                    self._unitId, self._modelId, self._tid_map = ids
+                    if _log.isEnabledFor(_INFO) and self._serial and self._serial != self._unitId:
+                        _log.info('%s: unitId %s does not match serial %s', self, self._unitId, self._serial)
         return self._unitId
 
     @property
     def modelId(self):
         if not self._modelId:
             if self.online and self.protocol >= 2.0:
-                self._unitId, self._modelId, self._tid_map = _hidpp20.get_ids(self)
+                ids = _hidpp20.get_ids(self)
+                if ids:
+                    self._unitId, self._modelId, self._tid_map = ids
+                    if _log.isEnabledFor(_INFO) and self._serial and self._serial != self._unitId:
+                        _log.info('%s: unitId %s does not match serial %s', self, self._unitId, self._serial)
         return self._modelId
 
     @property
     def tid_map(self):
         if not self._tid_map:
             if self.online and self.protocol >= 2.0:
-                self._unitId, self._modelId, self._tid_map = _hidpp20.get_ids(self)
+                ids = _hidpp20.get_ids(self)
+                if ids:
+                    self._unitId, self._modelId, self._tid_map = ids
+                    if _log.isEnabledFor(_INFO) and self._serial and self._serial != self._unitId:
+                        _log.info('%s: unitId %s does not match serial %s', self, self._unitId, self._serial)
         return self._tid_map
 
+    def update_pairing_information(self):
+        if self.receiver:
+            wpid, kind, polling_rate = self.receiver.device_pairing_information(self.number)
+            if not self.wpid:
+                self.wpid = wpid
+            if not self._kind:
+                self._kind = kind
+            if not self._polling_rate:
+                self._polling_rate = polling_rate
+
+    def update_extended_pairing_information(self):
+        if self.receiver:
+            serial, power_switch = self.receiver.device_extended_pairing_information(self.number)
+            if not self._serial:
+                self._serial = serial
+            if not self._power_switch:
+                self._power_switch = power_switch
+
     @property
     def kind(self):
         if not self._kind:
-            pair_info = self.receiver.read_register(_R.receiver_info, 0x20 + self.number - 1) if self.receiver else None
-            if pair_info:
-                kind = ord(pair_info[7:8]) & 0x0F
-                self._kind = _hidpp10.DEVICE_KIND[kind]
-            elif self.online and self.protocol >= 2.0:
-                self._kind = _hidpp20.get_kind(self)
+            self.update_pairing_information()
+            if not self._kind and self.protocol >= 2.0:
+                kind = _hidpp20.get_kind(self)
+                self._kind = KIND_MAP[kind] if kind else None
         return self._kind or '?'
 
     @property
@@ -259,42 +259,21 @@ class Device(object):
 
     @property
     def serial(self):
-        if not self._serial and self.receiver:
-            serial = self.receiver.read_register(_R.receiver_info, 0x30 + self.number - 1)
-            if serial:
-                ps = ord(serial[9:10]) & 0x0F
-                self._power_switch = _hidpp10.POWER_SWITCH_LOCATION[ps]
-            else:
-                # some Nano receivers?
-                serial = self.receiver.read_register(0x2D5)
-
-            if serial:
-                self._serial = _strhex(serial[1:5])
-            else:
-                # fallback...
-                self._serial = self.receiver.serial
+        if not self._serial:
+            self.update_extended_pairing_information()
         return self._serial or '?'
 
     @property
     def power_switch_location(self):
-        if not self._power_switch and self.receiver:
-            ps = self.receiver.read_register(_R.receiver_info, 0x30 + self.number - 1)
-            if ps:
-                ps = ord(ps[9:10]) & 0x0F
-                self._power_switch = _hidpp10.POWER_SWITCH_LOCATION[ps]
-            else:
-                self._power_switch = '(unknown)'
+        if not self._power_switch:
+            self.update_extended_pairing_information()
         return self._power_switch
 
     @property
     def polling_rate(self):
-        if not self._polling_rate and self.receiver:
-            pair_info = self.receiver.read_register(_R.receiver_info, 0x20 + self.number - 1)
-            if pair_info:
-                self._polling_rate = ord(pair_info[2:3])
-            else:
-                self._polling_rate = 0
-        if self.online and self.protocol >= 2.0 and self.features and _hidpp20.FEATURE.REPORT_RATE in self.features:
+        if not self._polling_rate:
+            self.update_pairing_information()
+        if not self._polling_rate and self.protocol >= 2.0:
             rate = _hidpp20.get_polling_rate(self)
             self._polling_rate = rate if rate else self._polling_rate
         return self._polling_rate
@@ -324,10 +303,9 @@ class Device(object):
 
     @property
     def settings(self):
-        if self._settings is None:
+        if not self._settings:
             self._settings = []
-            if self.descriptor and self.descriptor.settings and self.persister:
-                self._settings = []
+            if self.persister and self.descriptor and self.descriptor.settings:
                 for s in self.descriptor.settings:
                     try:
                         setting = s(self)
@@ -347,25 +325,6 @@ class Device(object):
             self._persister = _configuration.persister(self)
         return self._persister
 
-    def get_kind_from_index(self, index, receiver):
-        """Get device kind from 27Mhz device index"""
-        # accordingly to drivers/hid/hid-logitech-dj.c
-        # index 1 or 2 always mouse, index 3 always the keyboard,
-        # index 4 is used for an optional separate numpad
-
-        if index == 1:  # mouse
-            kind = 2
-        elif index == 2:  # mouse
-            kind = 2
-        elif index == 3:  # keyboard
-            kind = 1
-        elif index == 4:  # numpad
-            kind = 3
-        else:  # unknown device number on 27Mhz receiver
-            _log.error('failed to calculate device kind for device %d of %s', index, receiver)
-            raise _base.NoSuchDevice(number=index, receiver=receiver, error='Unknown 27Mhz device number')
-        return kind
-
     def enable_connection_notifications(self, enable=True):
         """Enable or disable device (dis)connection notifications on this
         receiver."""
@@ -408,8 +367,8 @@ class Device(object):
     def remove_notification_handler(self, id: str):
         """Unregisters the notification handler under name `id`."""
 
-        if id not in self._notification_handlers and _log.isEnabledFor(_WARNING):
-            _log.warn(f'Tried to remove nonexistent notification handler {id} from device {self}.')
+        if id not in self._notification_handlers and _log.isEnabledFor(_INFO):
+            _log.info(f'Tried to remove nonexistent notification handler {id} from device {self}.')
         else:
             del self._notification_handlers[id]
 
@@ -421,14 +380,16 @@ class Device(object):
         return None
 
     def request(self, request_id, *params, no_reply=False):
-        return _base.request(
-            self.handle or self.receiver.handle,
-            self.number,
-            request_id,
-            *params,
-            no_reply=no_reply,
-            long_message=self.bluetooth
-        )
+        if self:
+            return _base.request(
+                self.handle or self.receiver.handle,
+                self.number,
+                request_id,
+                *params,
+                no_reply=no_reply,
+                long_message=self.bluetooth or self.protocol >= 2.0,
+                protocol=self.protocol
+            )
 
     def feature_request(self, feature, function=0x00, *params, no_reply=False):
         if self.protocol >= 2.0:
@@ -436,7 +397,8 @@ class Device(object):
 
     def ping(self):
         """Checks if the device is online, returns True of False"""
-        protocol = _base.ping(self.handle or self.receiver.handle, self.number, long_message=self.bluetooth)
+        long = self.bluetooth or self._protocol is not None and self._protocol >= 2.0
+        protocol = _base.ping(self.handle or self.receiver.handle, self.number, long_message=long)
         self.online = protocol is not None
         if protocol:
             self._protocol = protocol
@@ -479,7 +441,7 @@ class Device(object):
         try:
             handle = _base.open_path(device_info.path)
             if handle:
-                return Device(None, 0, info=device_info)
+                return Device(None, 0xFF, info=device_info)
         except OSError as e:
             _log.exception('open %s', device_info)
             if e.errno == _errno.EACCES:
