@@ -1,5 +1,4 @@
 # -*- python-mode -*-
-# -*- coding: UTF-8 -*-
 
 ## Copyright (C) 2012-2013  Daniel Pavel
 ##
@@ -19,15 +18,15 @@
 
 # Logitech Unifying Receiver API.
 
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 from logging import DEBUG as _DEBUG
 from logging import ERROR as _ERROR
+from logging import INFO as _INFO
 from logging import WARNING as _WARNING
 from logging import getLogger
 from typing import List
 
 from . import special_keys
+from .common import BATTERY_APPROX as _BATTERY_APPROX
 from .common import FirmwareInfo as _FirmwareInfo
 from .common import KwException as _KwException
 from .common import NamedInt as _NamedInt
@@ -72,6 +71,7 @@ FEATURE = _NamedInts(
     DFU=0x00D0,
     BATTERY_STATUS=0x1000,
     BATTERY_VOLTAGE=0x1001,
+    UNIFIED_BATTERY=0x1004,
     CHARGING_CONTROL=0x1010,
     LED_CONTROL=0x1300,
     GENERIC_TEST=0x1800,
@@ -102,6 +102,7 @@ FEATURE = _NamedInts(
     POINTER_AXIS_ORIENTATION=0x2006,
     VERTICAL_SCROLLING=0x2100,
     SMART_SHIFT=0x2110,
+    SMART_SHIFT_ENHANCED=0x2111,
     HI_RES_SCROLLING=0x2120,
     HIRES_WHEEL=0x2121,
     LOWRES_WHEEL=0x2130,
@@ -111,6 +112,8 @@ FEATURE = _NamedInts(
     POINTER_SPEED=0x2205,
     ANGLE_SNAPPING=0x2230,
     SURFACE_TUNING=0x2240,
+    XY_STATS=0x2250,
+    WHEEL_STATS=0x2251,
     HYBRID_TRACKING=0x2400,
     # Keyboard
     FN_INVERSION=0x40A0,
@@ -158,6 +161,8 @@ FEATURE = _NamedInts(
     SIDETONE=0x8300,
     EQUALIZER=0x8310,
     HEADSET_OUT=0x8320,
+    # Fake features for Solaar internal use
+    MOUSE_GESTURE=0xFE00,
 )
 FEATURE._fallback = lambda x: 'unknown:%04X' % x
 
@@ -181,6 +186,8 @@ BATTERY_STATUS = _NamedInts(
     thermal_error=0x06
 )
 
+ONBOARD_MODES = _NamedInts(MODE_NO_CHANGE=0x00, MODE_ONBOARD=0x01, MODE_HOST=0x02)
+
 CHARGE_STATUS = _NamedInts(charging=0x00, full=0x01, not_charging=0x02, error=0x07)
 
 CHARGE_LEVEL = _NamedInts(average=50, full=90, critical=5)
@@ -219,7 +226,7 @@ class FeatureCallError(_KwException):
 #
 
 
-class FeaturesArray(object):
+class FeaturesArray:
     """A sequence of features supported by a HID++ 2.0 device."""
     __slots__ = ('supported', 'device', 'features', 'non_features')
     assert FEATURE.ROOT == 0x0000
@@ -256,7 +263,7 @@ class FeaturesArray(object):
 
             reply = self.device.request(0x0000, _pack('!H', FEATURE.FEATURE_SET))
             if reply is None:
-                self.supported = False
+                return False  # device might not be active so don't assume unsupported
             else:
                 fs_index = ord(reply[0:1])
                 if fs_index:
@@ -311,7 +318,7 @@ class FeaturesArray(object):
                 elif ivalue == int(f):
                     return True
 
-            if may_have:
+            if may_have and self.device:
                 reply = self.device.request(0x0000, _pack('!H', ivalue))
                 if reply:
                     index = ord(reply[0:1])
@@ -360,7 +367,7 @@ class FeaturesArray(object):
 #
 
 
-class ReprogrammableKey(object):
+class ReprogrammableKey:
     """Information about a control present on a device with the `REPROG_CONTROLS` feature.
     Ref: https://drive.google.com/file/d/0BxbRzx7vEV7eU3VfMnRuRXktZ3M/view
     Read-only properties:
@@ -517,13 +524,12 @@ class ReprogrammableKeyV4(ReprogrammableKey):
         """
         flags = flags if flags else {}  # See flake8 B006
 
-        if special_keys.MAPPING_FLAG.raw_XY_diverted in flags and flags[special_keys.MAPPING_FLAG.raw_XY_diverted]:
-            # We need diversion to report raw XY, so divert temporarily
-            # (since XY reporting is also temporary)
-            flags[special_keys.MAPPING_FLAG.diverted] = True
-
-        if special_keys.MAPPING_FLAG.diverted in flags and not flags[special_keys.MAPPING_FLAG.diverted]:
-            flags[special_keys.MAPPING_FLAG.raw_XY_diverted] = False
+        # if special_keys.MAPPING_FLAG.raw_XY_diverted in flags and flags[special_keys.MAPPING_FLAG.raw_XY_diverted]:
+        # We need diversion to report raw XY, so divert temporarily
+        # (since XY reporting is also temporary)
+        # flags[special_keys.MAPPING_FLAG.diverted] = True
+        # if special_keys.MAPPING_FLAG.diverted in flags and not flags[special_keys.MAPPING_FLAG.diverted]:
+        # flags[special_keys.MAPPING_FLAG.raw_XY_diverted] = False
 
         # The capability required to set a given reporting flag.
         FLAG_TO_CAPABILITY = {
@@ -563,7 +569,7 @@ class ReprogrammableKeyV4(ReprogrammableKey):
             )
         )
         ret = feature_request(self._device, FEATURE.REPROG_CONTROLS_V4, 0x30, *pkt)
-        if _unpack('!BBBBB', ret[:5]) != pkt and _log.isEnabledFor(_WARNING):
+        if ret is None or _unpack('!BBBBB', ret[:5]) != pkt and _log.isEnabledFor(_WARNING):
             _log.warn(
                 f"REPROG_CONTROLS_v4 endpoint setCidReporting on device {self._device} should echo request packet, but didn't."
             )
@@ -572,7 +578,7 @@ class ReprogrammableKeyV4(ReprogrammableKey):
         self._getCidReporting()
 
 
-class KeysArray(object):
+class KeysArray:
     """A sequence of key mappings supported by a HID++ 2.0 device."""
 
     __slots__ = ('device', 'keys', 'keyversion', 'cid_to_tid', 'group_cids')
@@ -807,7 +813,7 @@ ACTION_ID = _NamedInts(
 ACTION_ID._fallback = lambda x: 'unknown:%04X' % x
 
 
-class Gesture(object):
+class Gesture:
 
     gesture_index = {}
 
@@ -863,7 +869,7 @@ class Gesture(object):
     write = set
 
 
-class Param(object):
+class Param:
     param_index = {}
 
     def __init__(self, device, low, high):
@@ -941,7 +947,7 @@ class Spec:
         return f'[{self.spec}={self.value}]'
 
 
-class Gestures(object):
+class Gestures:
     """Information about the gestures that a device supports.
     Right now only some information fields are supported.
     WARNING: Assumes that parameters are always global, which is not the case.
@@ -1054,16 +1060,17 @@ def get_firmware(device):
 def get_ids(device):
     """Reads a device's ids (unit and model numbers)"""
     ids = feature_request(device, FEATURE.DEVICE_FW_VERSION)
-    unitId = ids[1:5]
-    modelId = ids[7:13]
-    transport_bits = ord(ids[6:7])
-    offset = 0
-    tid_map = {}
-    for transport, flag in [('btid', 0x1), ('btleid', 0x02), ('wpid', 0x04), ('usbid', 0x08)]:
-        if transport_bits & flag:
-            tid_map[transport] = modelId[offset:offset + 2].hex().upper()
-            offset = offset + 2
-    return (unitId.hex().upper(), modelId.hex().upper(), tid_map)
+    if ids:
+        unitId = ids[1:5]
+        modelId = ids[7:13]
+        transport_bits = ord(ids[6:7])
+        offset = 0
+        tid_map = {}
+        for transport, flag in [('btid', 0x1), ('btleid', 0x02), ('wpid', 0x04), ('usbid', 0x08)]:
+            if transport_bits & flag:
+                tid_map[transport] = modelId[offset:offset + 2].hex().upper()
+                offset = offset + 2
+        return (unitId.hex().upper(), modelId.hex().upper(), tid_map)
 
 
 def get_kind(device):
@@ -1130,14 +1137,31 @@ def get_battery(device):
     """Reads a device's battery level."""
     battery = feature_request(device, FEATURE.BATTERY_STATUS)
     if battery:
-        discharge, dischargeNext, status = _unpack('!BBB', battery[:3])
+        discharge, next, status = _unpack('!BBB', battery[:3])
         discharge = None if discharge == 0 else discharge
+        status = BATTERY_STATUS[status]
         if _log.isEnabledFor(_DEBUG):
-            _log.debug(
-                'device %d battery %s%% charged, next level %s%% charge, status %s = %s', device.number, discharge,
-                dischargeNext, status, BATTERY_STATUS[status]
-            )
-        return discharge, BATTERY_STATUS[status], dischargeNext
+            _log.debug('device %d battery %s%% charged, next %s%%, status %s', device.number, discharge, next, status)
+        return discharge, status, next
+    else:
+        battery = feature_request(device, FEATURE.UNIFIED_BATTERY, 0x10)
+        if battery:
+            return decipher_unified_battery(battery)
+
+
+def decipher_unified_battery(report):
+    discharge, level, status, _ignore = _unpack('!BBBB', report[:4])
+    status = BATTERY_STATUS[status]
+    if _log.isEnabledFor(_DEBUG):
+        _log.debug('battery %s%% charged, level %s, charging %s', discharge, level, status)
+    level = (
+        _BATTERY_APPROX.full if level == 8  # full
+        else _BATTERY_APPROX.good if level == 4  # good
+        else _BATTERY_APPROX.low if level == 2  # low
+        else _BATTERY_APPROX.critical if level == 1  # critical
+        else _BATTERY_APPROX.empty
+    )
+    return discharge if discharge else level, status, None
 
 
 def get_voltage(device):
@@ -1146,6 +1170,25 @@ def get_voltage(device):
         return decipher_voltage(battery_voltage)
 
 
+# voltage to remaining charge from Logitech
+battery_voltage_remaining = (
+    (4186, 100),
+    (4067, 90),
+    (3989, 80),
+    (3922, 70),
+    (3859, 60),
+    (3811, 50),
+    (3778, 40),
+    (3751, 30),
+    (3717, 20),
+    (3671, 10),
+    (3646, 5),
+    (3579, 2),
+    (3500, 0),
+    (-1000, 0),
+)
+
+
 # modified to be much closer to battery reports
 def decipher_voltage(voltage_report):
     voltage, flags = _unpack('>HB', voltage_report[:3])
@@ -1170,6 +1213,11 @@ def decipher_voltage(voltage_report):
     elif (flags & (1 << 5)):
         charge_lvl = CHARGE_LEVEL.critical
 
+    for level in battery_voltage_remaining:
+        if level[0] < voltage:
+            charge_lvl = level[1]
+            break
+
     if _log.isEnabledFor(_DEBUG):
         _log.debug(
             'device ???, battery voltage %d mV, charging = %s, charge status %d = %s, charge level %s, charge type %s',
@@ -1290,25 +1338,60 @@ def get_host_names(device):
         if capability_flags & 0x01:  # device can get host names
             for host in range(0, numHosts):
                 hostinfo = feature_request(device, FEATURE.HOSTS_INFO, 0x10, host)
-                _ignore, status, _ignore, numPages, nameLen, _ignore = _unpack('!BBBBBB', hostinfo[:6])
+                _ignore, status, _ignore, _ignore, nameLen, _ignore = _unpack('!BBBBBB', hostinfo[:6])
                 name = ''
                 remaining = nameLen
                 while remaining > 0:
                     name_piece = feature_request(device, FEATURE.HOSTS_INFO, 0x30, host, nameLen - remaining)
-                    name += name_piece[2:2 + min(remaining, 14)].decode()
-                    remaining = max(0, remaining - 14)
+                    if name_piece:
+                        name += name_piece[2:2 + min(remaining, 14)].decode()
+                        remaining = max(0, remaining - 14)
+                    else:
+                        remaining = 0
                 host_names[host] = (bool(status), name)
+        # update the current host's name if it doesn't match the system name
+        import socket
+        hostname = socket.gethostname().partition('.')[0]
+        if host_names[currentHost][1] != hostname:
+            set_host_name(device, hostname, host_names[currentHost][1])
+            host_names[currentHost] = (host_names[currentHost][0], hostname)
     return host_names
 
 
-def set_host_name(device, name):
+def set_host_name(device, name, currentName=''):
+    name = bytearray(name, 'utf-8')
+    currentName = bytearray(currentName, 'utf-8')
+    if _log.isEnabledFor(_INFO):
+        _log.info('Setting host name to %s', name)
     state = feature_request(device, FEATURE.HOSTS_INFO, 0x00)
     if state:
-        flags = _unpack('!B', state[:1])[0]
+        flags, _ignore, _ignore, currentHost = _unpack('!BBBB', state[:4])
         if flags & 0x02:
-            hn = name[:min(14, name.find('.'))] if name.find('.') >= 0 else name
-            response = feature_request(device, FEATURE.HOSTS_INFO, 0x40, 0xff, 0, hn)
-            return response
+            hostinfo = feature_request(device, FEATURE.HOSTS_INFO, 0x10, currentHost)
+            _ignore, _ignore, _ignore, _ignore, _ignore, maxNameLen = _unpack('!BBBBBB', hostinfo[:6])
+            if name[:maxNameLen] == currentName[:maxNameLen] and False:
+                return True
+            length = min(maxNameLen, len(name))
+            chunk = 0
+            while chunk < length:
+                response = feature_request(device, FEATURE.HOSTS_INFO, 0x40, currentHost, chunk, name[chunk:chunk + 14])
+                if not response:
+                    return False
+                chunk += 14
+        return True
+
+
+def get_onboard_mode(device):
+    state = feature_request(device, FEATURE.ONBOARD_PROFILES, 0x20)
+
+    if state:
+        mode = _unpack('!B', state[:1])[0]
+        return mode
+
+
+def set_onboard_mode(device, mode):
+    state = feature_request(device, FEATURE.ONBOARD_PROFILES, 0x10, mode)
+    return state
 
 
 def get_polling_rate(device):
@@ -1316,3 +1399,10 @@ def get_polling_rate(device):
     if state:
         rate = _unpack('!B', state[:1])[0]
         return rate
+
+
+def get_remaining_pairing(device):
+    result = feature_request(device, FEATURE.REMAINING_PAIRING, 0x0)
+    if result:
+        result = _unpack('!B', result[:1])[0]
+        return result
