@@ -1,5 +1,4 @@
 # -*- python-mode -*-
-# -*- coding: UTF-8 -*-
 
 ## Copyright (C) 2012-2013  Daniel Pavel
 ##
@@ -17,18 +16,20 @@
 ## with this program; if not, write to the Free Software Foundation, Inc.,
 ## 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 import math
 
 from copy import copy as _copy
 from logging import DEBUG as _DEBUG
+from logging import WARNING as _WARNING
 from logging import getLogger
 
+from . import hidpp20 as _hidpp20
 from .common import NamedInt as _NamedInt
 from .common import NamedInts as _NamedInts
 from .common import bytes2int as _bytes2int
 from .common import int2bytes as _int2bytes
+from .common import unpack as _unpack
+from .i18n import _
 
 _log = getLogger(__name__)
 del getLogger
@@ -37,10 +38,11 @@ del getLogger
 #
 #
 
+SENSITIVITY_IGNORE = 'ignore'
 KIND = _NamedInts(toggle=0x01, choice=0x02, range=0x04, map_choice=0x0A, multiple_toggle=0x10, multiple_range=0x40)
 
 
-class Setting(object):
+class Setting:
     """A setting descriptor.
     Needs to be instantiated for each specific device."""
     __slots__ = (
@@ -133,14 +135,14 @@ class Setting(object):
                 self._device.persister[self.name] = self._value
             return self._value
 
-    def _pre_write(self):
+    def _pre_write(self, save=True):
         # Remember the value we're trying to set, even if the write fails.
         # This way even if the device is offline or some other error occurs,
         # the last value we've tried to write is remembered in the configuration.
-        if self.persist and self._device.persister:
+        if self.persist and self._device.persister and save:
             self._device.persister[self.name] = self._value
 
-    def write(self, value):
+    def write(self, value, save=True):
         assert hasattr(self, '_value')
         assert hasattr(self, '_device')
         assert value is not None
@@ -149,8 +151,9 @@ class Setting(object):
             _log.debug('%s: settings write %r to %s', self.name, value, self._device)
 
         if self._device.online:
-            self._value = value
-            self._pre_write()
+            if self._value != value:
+                self._value = value
+                self._pre_write(save)
 
             current_value = None
             if self._validator.needs_current_value:
@@ -178,7 +181,11 @@ class Setting(object):
 
         value = self.read(self.persist)  # Don't use persisted value if setting doesn't persist
         if self.persist and value is not None:  # If setting doesn't persist no need to write value just read
-            self.write(value)
+            try:
+                self.write(value, save=False)
+            except Exception:
+                if _log.isEnabledFor(_WARNING):
+                    _log.warn('%s: error applying value %s so ignore it (%s)', self.name, self._value, self._device)
 
     def __str__(self):
         if hasattr(self, '_value'):
@@ -239,7 +246,7 @@ class Settings(Setting):
                 self._device.persister[self.name] = self._value
             return self._value[str(int(key))]
 
-    def write(self, map):
+    def write(self, map, save=True):
         assert hasattr(self, '_value')
         assert hasattr(self, '_device')
         assert map is not None
@@ -249,7 +256,7 @@ class Settings(Setting):
 
         if self._device.online:
             self._value = map
-            self._pre_write()
+            self._pre_write(save)
             for key, value in map.items():
                 data_bytes = self._validator.prepare_write(int(key), value)
                 if data_bytes is not None:
@@ -270,8 +277,11 @@ class Settings(Setting):
             _log.debug('%s: settings write key %r value %r to %s', self.name, key, value, self._device)
 
         if self._device.online:
+            if not self._value:
+                self.read()
             try:
                 data_bytes = self._validator.prepare_write(int(key), value)
+                # always need to write to configuration because dictionary is shared and could have changed
                 self._value[str(key)] = value
                 self._pre_write()
             except ValueError:
@@ -339,7 +349,7 @@ class LongSettings(Setting):
                 self._device.persister[self.name] = self._value
             return self._value[str(int(item))]
 
-    def write(self, map):
+    def write(self, map, save=True):
         assert hasattr(self, '_value')
         assert hasattr(self, '_device')
         assert map is not None
@@ -348,7 +358,7 @@ class LongSettings(Setting):
             _log.debug('%s: settings write %r to %s', self.name, map, self._device)
         if self._device.online:
             self._value = map
-            self._pre_write()
+            self._pre_write(save)
             for item, value in map.items():
                 data_bytes_list = self._validator.prepare_write(self._value)
                 if data_bytes_list is not None:
@@ -371,6 +381,8 @@ class LongSettings(Setting):
             _log.debug('%s: settings write item %r value %r to %s', self.name, item, value, self._device)
 
         if self._device.online:
+            if not self._value:
+                self.read()
             data_bytes = self._validator.prepare_write_item(item, value)
             self._value[str(int(item))] = value
             self._pre_write()
@@ -437,7 +449,7 @@ class BitFieldSetting(Setting):
     def _do_read_key(self, key):
         return self._rw.read(self._device, key)
 
-    def write(self, map):
+    def write(self, map, save=True):
         assert hasattr(self, '_value')
         assert hasattr(self, '_device')
         assert map is not None
@@ -446,7 +458,7 @@ class BitFieldSetting(Setting):
             _log.debug('%s: settings write %r to %s', self.name, map, self._device)
         if self._device.online:
             self._value = map
-            self._pre_write()
+            self._pre_write(save)
             data_bytes = self._validator.prepare_write(self._value)
             if data_bytes is not None:
                 if _log.isEnabledFor(_DEBUG):
@@ -469,6 +481,8 @@ class BitFieldSetting(Setting):
             _log.debug('%s: settings write key %r value %r to %s', self.name, key, value, self._device)
 
         if self._device.online:
+            if not self._value:
+                self.read()
             value = bool(value)
             self._value[str(key)] = value
             self._pre_write()
@@ -504,10 +518,10 @@ class BitFieldWithOffsetAndMaskSetting(BitFieldSetting):
 #
 
 
-class RegisterRW(object):
+class RegisterRW:
     __slots__ = ('register', )
 
-    kind = _NamedInt(0x01, 'register')
+    kind = _NamedInt(0x01, _('register'))
 
     def __init__(self, register):
         assert isinstance(register, int)
@@ -520,32 +534,36 @@ class RegisterRW(object):
         return device.write_register(self.register, data_bytes)
 
 
-class FeatureRW(object):
-    __slots__ = ('feature', 'read_fnid', 'write_fnid', 'no_reply')
+class FeatureRW:
+    __slots__ = ('feature', 'read_fnid', 'write_fnid', 'prefix', 'no_reply')
 
-    kind = _NamedInt(0x02, 'feature')
+    kind = _NamedInt(0x02, _('feature'))
     default_read_fnid = 0x00
     default_write_fnid = 0x10
+    default_prefix = b''
 
-    def __init__(self, feature, read_fnid=default_read_fnid, write_fnid=default_write_fnid, no_reply=False):
+    def __init__(
+        self, feature, read_fnid=default_read_fnid, write_fnid=default_write_fnid, prefix=default_prefix, no_reply=False
+    ):
         assert isinstance(feature, _NamedInt)
         self.feature = feature
         self.read_fnid = read_fnid
         self.write_fnid = write_fnid
         self.no_reply = no_reply
+        self.prefix = prefix
 
     def read(self, device, data_bytes=b''):
         assert self.feature is not None
-        return device.feature_request(self.feature, self.read_fnid, data_bytes)
+        return device.feature_request(self.feature, self.read_fnid, self.prefix, data_bytes)
 
     def write(self, device, data_bytes):
         assert self.feature is not None
-        reply = device.feature_request(self.feature, self.write_fnid, data_bytes, no_reply=self.no_reply)
+        reply = device.feature_request(self.feature, self.write_fnid, self.prefix, data_bytes, no_reply=self.no_reply)
         return reply if not self.no_reply else True
 
 
 class FeatureRWMap(FeatureRW):
-    kind = _NamedInt(0x02, 'feature')
+    kind = _NamedInt(0x02, _('feature'))
     default_read_fnid = 0x00
     default_write_fnid = 0x10
     default_key_byte_count = 1
@@ -583,8 +601,8 @@ class FeatureRWMap(FeatureRW):
 #
 
 
-class BooleanValidator(object):
-    __slots__ = ('true_value', 'false_value', 'mask', 'needs_current_value')
+class BooleanValidator:
+    __slots__ = ('true_value', 'false_value', 'read_skip_byte_count', 'write_prefix_bytes', 'mask', 'needs_current_value')
 
     kind = KIND.toggle
     default_true = 0x01
@@ -592,7 +610,14 @@ class BooleanValidator(object):
     # mask specifies all the affected bits in the value
     default_mask = 0xFF
 
-    def __init__(self, true_value=default_true, false_value=default_false, mask=default_mask):
+    def __init__(
+        self,
+        true_value=default_true,
+        false_value=default_false,
+        mask=default_mask,
+        read_skip_byte_count=0,
+        write_prefix_bytes=b''
+    ):
         if isinstance(true_value, int):
             assert isinstance(false_value, int)
             if mask is None:
@@ -626,8 +651,11 @@ class BooleanValidator(object):
         self.true_value = true_value
         self.false_value = false_value
         self.mask = mask
+        self.read_skip_byte_count = read_skip_byte_count
+        self.write_prefix_bytes = write_prefix_bytes
 
     def validate_read(self, reply_bytes):
+        reply_bytes = reply_bytes[self.read_skip_byte_count:]
         if isinstance(self.mask, int):
             reply_value = ord(reply_bytes[:1]) & self.mask
             if _log.isEnabledFor(_DEBUG):
@@ -670,6 +698,7 @@ class BooleanValidator(object):
                 to_write |= ord(current_value[:1]) & (0xFF ^ self.mask)
             if current_value is not None and to_write == ord(current_value[:1]):
                 return None
+            to_write = bytes([to_write])
         else:
             to_write = bytearray(to_write)
             count = len(self.mask)
@@ -689,10 +718,10 @@ class BooleanValidator(object):
         if _log.isEnabledFor(_DEBUG):
             _log.debug('BooleanValidator: prepare_write(%s, %s) => %r', new_value, current_value, to_write)
 
-        return to_write
+        return self.write_prefix_bytes + to_write
 
 
-class BitFieldValidator(object):
+class BitFieldValidator:
     __slots__ = ('byte_count', 'options')
 
     kind = KIND.multiple_toggle
@@ -709,7 +738,7 @@ class BitFieldValidator(object):
         r = _bytes2int(reply_bytes[:self.byte_count])
         value = {str(int(k)): False for k in self.options}
         m = 1
-        for _ in range(8 * self.byte_count):
+        for _ignore in range(8 * self.byte_count):
             if m in self.options:
                 value[str(int(m))] = bool(r & m)
             m <<= 1
@@ -727,7 +756,7 @@ class BitFieldValidator(object):
         return self.options
 
 
-class BitFieldWithOffsetAndMaskValidator(object):
+class BitFieldWithOffsetAndMaskValidator:
     __slots__ = ('byte_count', 'options', '_option_from_key', '_mask_from_offset', '_option_from_offset_mask')
 
     kind = KIND.multiple_toggle
@@ -783,7 +812,7 @@ class BitFieldWithOffsetAndMaskValidator(object):
             value = _bytes2int(b[:self.byte_count])
             mask_to_opt = self._option_from_offset_mask.get(offset, {})
             m = 1
-            for _ in range(8 * self.byte_count):
+            for _ignore in range(8 * self.byte_count):
                 if m in mask_to_opt:
                     values[str(int(mask_to_opt[m]))] = bool(value & m)
                 m <<= 1
@@ -809,13 +838,13 @@ class BitFieldWithOffsetAndMaskValidator(object):
         return [int(opt) if isinstance(opt, int) else opt.as_int() for opt in self.options]
 
 
-class ChoicesValidator(object):
+class ChoicesValidator:
     kind = KIND.choice
     """Translates between NamedInts and a byte sequence.
     :param choices: a list of NamedInts
     :param byte_count: the size of the derived byte sequence. If None, it
     will be calculated from the choices."""
-    def __init__(self, choices, byte_count=None, read_skip_byte_count=None, write_prefix_bytes=b''):
+    def __init__(self, choices, byte_count=None, read_skip_byte_count=0, write_prefix_bytes=b''):
         assert choices is not None
         assert isinstance(choices, _NamedInts)
         assert len(choices) > 1
@@ -828,7 +857,7 @@ class ChoicesValidator(object):
             assert self._byte_count <= byte_count
             self._byte_count = byte_count
         assert self._byte_count < 8
-        self._read_skip_byte_count = read_skip_byte_count if read_skip_byte_count else 0
+        self._read_skip_byte_count = read_skip_byte_count
         self._write_prefix_bytes = write_prefix_bytes if write_prefix_bytes else b''
         assert self._byte_count + self._read_skip_byte_count <= 14
         assert self._byte_count + len(self._write_prefix_bytes) <= 14
@@ -868,7 +897,9 @@ class ChoicesMapValidator(ChoicesValidator):
         byte_count=None,
         read_skip_byte_count=0,
         write_prefix_bytes=b'',
-        extra_default=None
+        extra_default=None,
+        mask=-1,
+        activate=0
     ):
         assert choices_map is not None
         assert isinstance(choices_map, dict)
@@ -892,15 +923,17 @@ class ChoicesMapValidator(ChoicesValidator):
         self.choices = choices_map
         self.needs_current_value = False
         self.extra_default = extra_default
-        self._read_skip_byte_count = read_skip_byte_count if read_skip_byte_count else 0
+        self._read_skip_byte_count = read_skip_byte_count
         self._write_prefix_bytes = write_prefix_bytes if write_prefix_bytes else b''
+        self.activate = activate
+        self.mask = mask
         assert self._byte_count + self._read_skip_byte_count + self._key_byte_count <= 14
         assert self._byte_count + len(self._write_prefix_bytes) + self._key_byte_count <= 14
 
     def validate_read(self, reply_bytes, key):
         start = self._key_byte_count + self._read_skip_byte_count
         end = start + self._byte_count
-        reply_value = _bytes2int(reply_bytes[start:end])
+        reply_value = _bytes2int(reply_bytes[start:end]) & self.mask
         # reprogrammable keys starts out as 0, which is not a choice, so don't use assert here
         if self.extra_default is not None and self.extra_default == reply_value:
             return int(self.choices[key][0])
@@ -909,13 +942,15 @@ class ChoicesMapValidator(ChoicesValidator):
         return reply_value
 
     def prepare_write(self, key, new_value):
-        choices = self.choices[key]
-        if new_value not in choices and new_value != self.extra_default:
-            raise ValueError('invalid choice %r' % new_value)
+        choices = self.choices.get(key)
+        if choices is None or (new_value not in choices and new_value != self.extra_default):
+            _log.error('invalid choice %r for %s', new_value, key)
+            return None
+        new_value = new_value | self.activate
         return self._write_prefix_bytes + new_value.to_bytes(self._byte_count, 'big')
 
 
-class RangeValidator(object):
+class RangeValidator:
     __slots__ = ('min_value', 'max_value', 'flag', '_byte_count', 'needs_current_value')
 
     kind = KIND.range
@@ -1016,3 +1051,94 @@ class MultipleRangeValidator:
                 raise ValueError(f'invalid choice for {item}.{sub_item}: {v} not in [{sub_item.minimum}..{sub_item.maximum}]')
             w += _int2bytes(v, sub_item.length)
         return w + b'\xFF'
+
+
+class ActionSettingRW:
+    """Special RW class for settings that turn on and off special processing when a key or button is depressed"""
+    def __init__(self, name, divert_setting_name):
+        self.name = name
+        self.divert_setting_name = divert_setting_name
+        self.kind = FeatureRW.kind  # pretend to be FeatureRW as required for HID++ 2.0 devices
+        self.device = None
+        self.key = None
+        self.active = False
+        self.pressed = False
+
+    def activate_action(self):  # action to take when setting is activated (write non-false)
+        pass
+
+    def deactivate_action(self):  # action to take when setting is deactivated (write false)
+        pass
+
+    def press_action(self):  # action to take when key is pressed
+        pass
+
+    def release_action(self):  # action to take when key is released
+        pass
+
+    def move_action(self, dx, dy):  # action to take when mouse is moved while key is down
+        pass
+
+    def key_action(self, key):  # acction to take when some other diverted key is pressed
+        pass
+
+    def read(self, device):  # need to return bytes, as if read from device
+        return _int2bytes(self.key, 2) if self.active else b'\x00\x00'
+
+    def write(self, device, data_bytes):
+        def handler(device, n):  # Called on notification events from the device
+            if n.sub_id < 0x40 and device.features[n.sub_id] == _hidpp20.FEATURE.REPROG_CONTROLS_V4:
+                if n.address == 0x00:
+                    cids = _unpack('!HHHH', n.data[:8])
+                    if not self.pressed and int(self.key.key) in cids:  # trigger key pressed
+                        self.pressed = True
+                        self.press_action()
+                    elif self.pressed:
+                        if int(self.key.key) not in cids:  # trigger key released
+                            self.pressed = False
+                            self.release_action()
+                        else:
+                            for key in cids:
+                                if key and not key == self.key.key:  # some other diverted key pressed
+                                    self.key_action(key)
+                elif n.address == 0x10:
+                    if self.pressed:
+                        dx, dy = _unpack('!hh', n.data[:4])
+                        self.move_action(dx, dy)
+
+        divertSetting = next(filter(lambda s: s.name == self.divert_setting_name, device.settings), None)
+        self.device = device
+        key = _bytes2int(data_bytes)
+        if key:  # Enable
+            self.key = next((k for k in device.keys if k.key == key), None)
+            if self.key:
+                self.active = True
+                divertSetting.write_key_value(int(self.key.key), 1)
+                device.add_notification_handler(self.name, handler)
+                from solaar.ui import status_changed as _status_changed
+                self.activate_action()
+                _status_changed(device, refresh=True)  # update main window
+            else:
+                _log.error('cannot enable %s on %s for key %s', self.name, device, key)
+        else:  # Disable
+            if self.active:
+                self.active = False
+                divertSetting.write_key_value(int(self.key.key), 0)
+                from solaar.ui import status_changed as _status_changed
+                _status_changed(device, refresh=True)  # update main window
+                try:
+                    device.remove_notification_handler(self.name)
+                except Exception:
+                    if _log.isEnabledFor(_WARNING):
+                        _log.warn('cannot disable %s on %s', self.name, device)
+                self.deactivate_action()
+        return True
+
+
+def apply_all_settings(device):
+    persister = getattr(device, 'persister', None)
+    sensitives = persister.get('_sensitive', {}) if persister else {}
+    for s in device.settings:
+        ignore = sensitives.get(s.name, False)
+        if ignore != SENSITIVITY_IGNORE:
+            s.apply()
