@@ -1,7 +1,7 @@
 # -*- python-mode -*-
-# -*- coding: UTF-8 -*-
 
 ## Copyright (C) 2012-2013  Daniel Pavel
+## Revisions Copyright (C) Contributors to the Solaar project.
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
 ## with this program; if not, write to the Free Software Foundation, Inc.,
 ## 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 from gi.repository import Gtk
 from solaar import NAME, __version__
 from solaar.i18n import _
@@ -35,15 +33,15 @@ def _create():
 
     about.set_program_name(NAME)
     about.set_version(__version__)
-    about.set_comments(_('Shows status of devices connected\nthrough wireless Logitech receivers.'))
-
+    about.set_comments(_('Manages Logitech receivers,\nkeyboards, mice, and tablets.'))
     about.set_logo_icon_name(NAME.lower())
 
-    about.set_copyright('© 2012-2013 Daniel Pavel')
+    about.set_copyright('© 2012-2021 Daniel Pavel and contributors to the Solaar project')
     about.set_license_type(Gtk.License.GPL_2_0)
 
     about.set_authors(('Daniel Pavel http://github.com/pwr', ))
     try:
+        about.add_credit_section(_('Additional Programming'), ('Filipe Laíns', 'Peter F. Patel-Schneider'))
         about.add_credit_section(_('GUI design'), ('Julien Gascard', 'Daniel Pavel'))
         about.add_credit_section(
             _('Testing'), (
@@ -80,7 +78,7 @@ def _create():
         ))
     )
 
-    about.set_website('http://pwr-solaar.github.io/Solaar/')
+    about.set_website('https://pwr-solaar.github.io/Solaar')
     about.set_website_label(NAME)
 
     about.connect('response', lambda x, y: x.hide())
